@@ -9,6 +9,18 @@ use base qw( Alien::Base::ModuleBuild );
 
 my $libffi_version = '3.2.1';
 
+# some weird older perl CPAN.pm or Module::Build or something.
+# installs okay with older versions of Alien::Base
+# even though 0.009 is a configure_requires
+# we need to nip it in the bud here else
+# it breaks in FFI::Platypus
+if($] < 5.010 && ! eval { use Alien::Base 0.009 (); 1 })
+{
+  print STDERR "Alien::Base 0.009 really is required as a configure prereq.\n";
+  print STDERR "Please install it first.\n";
+  exit;
+}
+
 sub new
 {
   my($class, %args) = @_;
@@ -39,8 +51,6 @@ sub new
       }
       # TODO: also need to copy libffi.a => ffi.lib
     }
-    
-    $args{build_requires}->{'Alien::MSYS'} = 0;
   }
   
   $args{alien_name} = 'libffi';
@@ -57,33 +67,23 @@ sub new
   };
   
   my $self = $class->SUPER::new(%args);
+
+  if($^O eq 'MSWin32' && ($Alien::Base::ModuleBuild::Force || $self->alien_check_installed_version))
+  {
+    $self->_add_prereq( 'build_requires' => 'Alien::MSYS' => 0 );
+  }
   
   $self;
 }
 
-sub do_system
+*do_system = sub
 {
   my($self, @rest) = @_;
-  
-  if($^O eq 'MSWin32')
-  {
-    require Alien::MSYS;
-    my $msys_path = Alien::MSYS::msys_path();
-    @PATH = grep { $_ ne $msys_path && $_ !~ /GnuWin32/i } @PATH;
-    unshift @PATH, $msys_path;  
-  }
-  
-  if(ref($rest[0]) eq 'ARRAY')
-  {
-    my @cmd = map { $self->alien_interpolate($_) } @{ $rest[0] };
-    print "+ @cmd\n";
-    system @cmd;
-    return ( success => ($? == 0), command => "@cmd" );
-  }
-  else
-  {
-    $self->SUPER::do_system(@rest);
-  }
-};
+  require Alien::MSYS;
+  my $msys_path = Alien::MSYS::msys_path();
+  @PATH = grep { $_ ne $msys_path && $_ !~ /GnuWin32/i } @PATH;
+  unshift @PATH, $msys_path;
+  $self->SUPER::do_system(@rest);
+} if $^O eq 'MSWin32';
 
 1;
